//
//  ImageFilterOperation.swift
//  Concurrency
//
//  Created by Valery Bashkatov on 07.02.16.
//  Copyright © 2016 Valery Bashkatov. All rights reserved.
//

import UIKit

class ImageFilterOperation: ImageOperation {
    // MARK: - Properties
    private var picture: Picture!
    
    // MARK: - Initialization
    init(picture: Picture) {
        super.init()
        self.picture = picture
    }
    
    override func start() {
        executing = true
        
        // If operation is cancelled, stop process
        guard !cancelled else {
            executing = false
            finished = true
            return
        }
        
        // Make filtered image
        let inputImage = CIImage(image: picture.originalImage)
        
        let context = CIContext(EAGLContext: EAGLContext(API: EAGLRenderingAPI.OpenGLES2))
        let filter = CIFilter(name: "CIPhotoEffectMono")!
        filter.setValue(inputImage, forKey: kCIInputImageKey)
        let outputImage = filter.outputImage
        let outputCGImage = context.createCGImage(outputImage!, fromRect: outputImage!.extent)
        
        guard !cancelled else {
            executing = false
            finished = true
            return
        }
        
        picture.filteredImage = UIImage(CGImage: outputCGImage)
        
        executing = false
        finished = true
    }
}
