//
//  ViewController.swift
//  Concurrency
//
//  Created by Valery Bashkatov on 27.01.16.
//  Copyright © 2016 Valery Bashkatov. All rights reserved.
//

import UIKit

class ViewController: UIViewController {
    let queue = NSOperationQueue()
    
    let picture = Picture(url:
        "http://valery.bashkatov.org/images/concurrent-programming/sample.jpg")
    
    let originalImageView = UIImageView(frame: CGRect(x: 0, y: 0, width: 300, height: 200))
    let filteredImageView = UIImageView(frame: CGRect(x: 0, y: 200, width: 300, height: 200))
    
    override func viewDidLoad() {
        super.viewDidLoad()

        view.addSubview(originalImageView)
        view.addSubview(filteredImageView)
        
        /*
        Create operations with dependencies: originalImageDownload - originalImageShow
        |
        originalImageFilter - filteredImageShow
        */
        let originalImageDownload = ImageDownloadOperation(picture: picture)
        
        let originalImageShow = NSBlockOperation {
            NSOperationQueue.mainQueue().addOperationWithBlock {
                self.originalImageView.image = self.picture.originalImage
            }
        }
        originalImageShow.addDependency(originalImageDownload)
        
        let originalImageFilter = ImageFilterOperation(picture: picture)
        originalImageFilter.addDependency(originalImageDownload)
        
        let filteredImageShow = NSBlockOperation {
            NSOperationQueue.mainQueue().addOperationWithBlock {
                self.filteredImageView.image = self.picture.filteredImage
            }
        }
        filteredImageShow.addDependency(originalImageFilter)
        
        queue.addOperations([originalImageDownload,
                             originalImageShow,
                             originalImageFilter,
                             filteredImageShow], waitUntilFinished: false)
    }
}